/*
Контроллер, MQTT-клиент для ioBroker, работа через адаптер MQTT (тек. версия 0.2.4)
IP address: 192.168.69.71
Platform: Arduino UNO R3 + EthernetShield W5100
IDE: 1.0.5-r2
Date: 15.05.2015
Version: 0.1
*****
Для тестирования опрашиваются аналоговые входы pinA0, pinA1, pinA2. Значения типа int (АЦП 0...1023).
ID клиента, как и корневая папка "arduinoPLC71".
При старте автоматически создаются топики:
1. "arduinoPLC71/status" - состояние контроллера (online, error)
2. "arduinoPLC71/pollTimer" - интервал публикации переменных (клиент подписан на это значение)
3. "arduinoPLC71/sensorA0/value" - группа sensorA0, результат измерения
4. "arduinoPLC71/sensorA0/status" - группа sensorA0, состояние датчика (ОК, error)
5. аналогично для групп sensorA1 и sensorA2.
При старте настраивается сетевое соединение (MAC-адрес и статический IP адрес), инициализируется отладочный
порт и контроллер подключается к MQTT-брокеру в качестве клиента. Во время работы основного цикла проверяется
состояние соединения с MQTT-боркером, если оно прервано, контроллер пытается его восстановить.
Данные публикуются после их обновления, необходимо использовать обработчик ошибок, при неверных данных
публиковать ошибку (или её код) в топик "arduinoPLC71/sensorA0/status" (для группы sensorA0, к примеру).
**/
//------------------------------------------ Libraries ------------------------------------------//
//Подключаем библиотеки
#include <SPI.h>
#include <Ethernet.h>
#include <PubSubClient.h> //https://github.com/knolleary/pubsubclient
//---------------------------------------- End libraries ----------------------------------------//

//------------------------------------------- Ethernet ------------------------------------------//
//Глобальные константы для работы с сетью
byte mac[]    = {  0xDE, 0xED, 0xBA, 0xFE, 0xFE, 0x71 };
byte mqttserver[] = { 192, 168, 69, 51 };
byte ip[]     = { 192, 168, 69, 71 };
EthernetClient ethClient;
PubSubClient client(mqttserver, 1883, callback, ethClient);
//----------------------------------------- End ethernet ----------------------------------------//

//------------------------------------------- Sensors -------------------------------------------//
int analogPinA0 = 0;
int analogPinA1 = 1;
int analogPinA2 = 2;
int SensorA0 = 0;
int SensorA1 = 0;
int SensorA2 = 0;
char buff[20];
unsigned long lastTime = 0;
unsigned long poolTime = 5; // по-умолчанию интервал опроса 5 сек
//----------------------------------------- End Sensors -----------------------------------------//

//###############################################################################################//
//########################################## Functions ##########################################//
//###############################################################################################//

//------------------------------------------- Callback -----------------------------------------//
//Функция обработки входящих соединений - прием данных по подписке
void callback(char* topic, byte* payload, unsigned int length) {
  Serial.println ("");
  Serial.println ("-------");
  Serial.println ("New callback of MQTT-broker");
  //преобразуем тему(topic) и значение (payload) в строку
  payload[length] = '\0';
  String strTopic = String(topic);
  String strPayload = String((char*)payload);
  //значение целлочисленное, преобразем к типу данных int
  poolTime = strPayload.toInt();
  Serial.print (strTopic);
  Serial.print (" ");
  Serial.println (poolTime);
  Serial.println ("-------");
  Serial.println ("");  
}
//----------------------------------------- End callback ---------------------------------------//

//------------------------------------------ Void setup ----------------------------------------//
void setup() {
  Serial.begin(9600);
  Serial.println("Start mqtt-client controller arduinoPLC7...");
  //стартуем сетевое подключение
  Ethernet.begin(mac, ip);
  Serial.print("PLC's IP: ");
  Serial.println(Ethernet.localIP());
  Serial.print("Connect to MQTT-boker...  ");
  //подключаемся к MQTT-брокеру
  if (client.connect("arduinoPLC71")) {
    Serial.println("success");
    client.publish("arduinoPLC71/status","online");
    client.subscribe("arduinoPLC71/pollTimer");
  }
  else {
    Serial.println("failed!");
  }
}
//---------------------------------------- End void setup --------------------------------------//

//------------------------------------------ Void loop -----------------------------------------//
void loop() {
  client.loop(); //проверка входящих соединений по подписке
  //если соединение не активно
  if (!client.connected()) {
    Serial.print("Reconnect to MQTT-boker...  ");
    //пытаемся его восстановить
    if (client.connect("arduinoPLC71")) {
      Serial.println("success");
      client.publish("arduinoPLC71/status","online");
      client.subscribe("arduinoPLC71/pollTimer");
    }
    else {
      Serial.println("failed!");
    }
  }
  //проверяем условие интервала измерений
  if(millis() > (lastTime + poolTime*1000)) {
    lastTime = millis();
    //если соединение активно
    if (client.connected()) {
      Serial.println(" ");
      //считываем данные и публикуем их
      SensorA0 = analogRead(analogPinA0); 
      sprintf(buff,"%d",SensorA0); //if float dtostrf(SensorA0, 5, 2, buff);
      Serial.print("Publish to MQTT-broker value sensor A0 is: ");
      Serial.println(buff);
      client.publish("arduinoPLC71/sensorA0/value",buff);
      client.publish("arduinoPLC71/sensorA0/status","OK");
      SensorA1 = analogRead(analogPinA1); 
      sprintf(buff,"%d",SensorA1); //if float dtostrf(SensorA0, 5, 2, buff);
      Serial.print("Publish to MQTT-broker value sensor A1 is: ");
      Serial.println(buff);
      client.publish("arduinoPLC71/sensorA1/value",buff);
      client.publish("arduinoPLC71/sensorA1/status","OK");
      SensorA2 = analogRead(analogPinA2); 
      sprintf(buff,"%d",SensorA2); //if float dtostrf(SensorA0, 5, 2, buff);
      Serial.print("Publish to MQTT-broker value sensor A2 is: ");
      Serial.println(buff);
      client.publish("arduinoPLC71/sensorA2/value",buff);
      client.publish("arduinoPLC71/sensorA2/status","OK");
      //--------
    }
  }
}
//---------------------------------------- End void loop ---------------------------------------//
